#include "cl/FileName.h"
#include "cl/List.h"
#include <stdlib.h>
#include "cl/Hash.h"
#include "cl/Heap.h"

#define CL_INST_RW
#include "cl/InstArray.h"

class StringNode : public Node, public String
{
public:
	StringNode() : Node(), String() {};
	StringNode(const char *s) : Node(), String() { (String&) SELF = s; };
	
	static Void *New() { return new StringNode; };
};

Void TestList()
{
	TypedList<StringNode>	list;
	StringNode tt, *temp;

	cout << "\nList Test" << endl;
	cout << "---------\n" << endl;

	(String&) tt = "red";
	cout << list << endl;
	list.Append(&tt);
	list.Append(new StringNode("orange"));
	temp = (StringNode*) StringNode::New();
	*temp = "green";
	list.Append(temp);
	cout << list << endl;
}

Void MyPrint(StrConst bob)
{
	cout << "print(StrConst): " << bob << endl;
}

Void TestString()
{
	String		string;
	StrConst 	strconst = "strconst";
	Int			i;
	StrConstArray	a;
	
	string = "string";
	
	cout << "\nString Test" << endl;
	cout << "-----------\n" << endl;

	cout << "sprintf test: " 
		 << String().Printf("number %d, strings '%s' '%s' '%s'", 
		 	20, "cstring", strconst, string.CString()) << endl;

	cout << "StrConst + CStr: " << strconst + "cstr" << endl;
	cout << "String + CStr: " << string + "cstr" << endl;
	cout << "CStr + String: " << ("cstr" + string) << endl;
	cout << "String + StrConst + CStr: " << string + strconst + "cstr" << endl;

	MyPrint(string);
	MyPrint("blah-" + string + "-blah");

	cout << "iso for string/strconst == ";
	if (string == string && strconst == strconst)
		cout << "good" << endl;
	else
		cout << "bad" << endl;

	cout << "string/cstr == ";
	if (string == "string" && "string" == string)
		cout << "good" << endl;
	else
		cout << "bad" << endl;

	cout << "strconst/cstr == ";
	if (strconst == "strconst" && "strconst" == strconst)
		cout << "good" << endl;
	else
		cout << "bad" << endl;

	string = strconst;
	cout << "string/strconst == ";
	if (strconst == string && string == strconst)
		cout << "good" << endl;
	else
		cout << "bad" << endl;

	string = "abcdefgh";

	cout << "Suffix/Prefix test: " << string.Suffix(2) << ':' << string.Suffix(-2) << ':'
		 << string.Prefix(2) << ':' << string.Prefix(-2) << '<' << endl;

	cout << "Suffix/Prefix bounds test: " << string.Suffix(10) << ':' << string.Suffix(-10) << ':'
		 << string.Prefix(10) << ':' << string.Prefix(-10) << '<' << endl;

	cout << "SubString test: " << string.SubString(2, 4) << ':' << string.SubString(-6, 4) << '<' << endl;

	cout << "SubString bounds test: " << string.SubString(10, 4) << ':' << string.SubString(-10, 4)
		 << ':' << string.SubString(2, 10) << '<' << endl;

	cout << "Find char test: " << string.FindChar('c')
		 << ", " << string.FindCharLast('x')
		 << ", " << strconst.FindChar('s')
		 << ", " << strconst.FindCharLast('s') << endl;

	cout << "Find tokens test: " << endl;
	string = "	 there is a disease   that is tearing apart the  lives of thousands";
	cout << "string = " << string << endl;
	Split(string, a);
	for (i = 0; i < a.NumItems(); i++)
		cout << "token " << i << " = '" << a[i] << "'" << endl;
}

static Void TestLoad(const FileName &fname)
{
	cout << "exists: " << fname.GetPath() << ": " << fname.IsReadable() << endl;
}

StrConst kExt[] =
{
	"a",
	"b",
	0
};

Void TestFileName()
{
	FileName	t;

	cout << "\nFileName Test" << endl;
	cout << "-------------\n" << endl;

	t.SetPath("/one/two/three/and/wibble.sl.gz");	

	cout << "the path is " << t.GetPath() << endl;
	cout << "parent: " << t.GetDir() << endl;
	cout << "name: " << t.GetFile() << endl;
	cout << "extensions: " << t.GetExtension() << endl;
	t.RemoveExtension();
	cout << "removed extension" << endl;
	cout << "new name: " << t.GetFile() << endl;
	cout << "new ext: " << t.GetExtension() << endl;

	TestLoad(t);
	TestLoad(FileName().SetPath("Makefile"));
	TestLoad(FileName().SetPath("Makefile.depend"));

	FileName fname;

	Int x;

	x = fname.SetPath("a").FindFileExtension(kExt);
	cout << String().Printf("a: file %s %d [%d]\n", fname.GetPath().CString(), fname.flags, x);

	x = fname.SetPath("a.b").FindFileExtension(kExt);
	cout << String().Printf("a.b.gz: file %s %d [%d]\n", fname.GetPath().CString(), fname.flags, x);

	x = fname.SetPath("a.e").FindFileExtension(kExt);
	cout << String().Printf("a.e: file %s %d [%d]\n", fname.GetPath().CString(), fname.flags, x);

	x = fname.SetPath("b").FindFileExtension(kExt);
	cout << String().Printf("b: file %s %d [%d]\n", fname.GetPath().CString(), fname.flags, x);
}

Void TestEnviron()
{
	cout << "\nEnviron Test" << endl;
	cout << "------------\n" << endl;

	cout << SubstituteEnvVars("shell = $SHELL, path = $HOME/bob") << endl;
}

Void TestInstArray()
{
	InstArray<Int> 	a;
	Int				i, sum;
	
	cout << "\nInstArray Test" << endl;
	cout << "--------------\n" << endl;

	a.stats.StartTrace(FileName().SetPath("trace.data"));
	a.SetSize(20);
	a.stats.accesses = new Int[a.NumItems()];
	for (i = 0; i < a.NumItems(); i++)
		a.stats.accesses[i] = 0;
		
	for (i = 0; i < a.NumItems(); i++)
		a[i] = i;

	for (i = 0; i < a.NumItems(); i++)
		a[i] += 3;

	for (i = 0; i < a.NumItems(); i++)
		sum += a[i];

	for (i = 0; i < 100; i++)
		a[rand() % 20] += 21;
		
	a.stats.StopTrace();
	a.stats.Dump();
	for (i = 0; i < a.NumItems(); i++)
		cout << i << " = " << a.stats.accesses[i] << endl;
}

class Printer : public IntHashIter
{
public:
	Void ProcessItem(StrConst s, Int a)
	{ cout << s << " -> " << a << endl; };
};

Void TestHash()
{
	cout << "\nHash Test" << endl;
	cout << "---------\n" << endl;

	IntHash		hash;
	Printer		printer;
	
	hash.SetItem("bob", 20);
	hash.SetItem("mary", 33);
	hash.SetItem("peter", 21);
	hash.SetItem("jane", 10);

	hash.SetItem("mary", hash.GetItem("mary") + 1);
	
	hash.Iterate(printer);

	cout << "bob exists: " << hash.ItemExists("bob") << endl;
	cout << "robert exists: " << hash.ItemExists("robert") << endl;
}

Void TestHeap()
{
	Heap		heap;
	HeapEntry 	*he;
	Int			i, n = 100;
	
	srand48(20);
	
	for (i = 0; i < n; i++)
	{
		he = new HeapEntry;

		he->cost = drand48();
		heap.Insert(he);
	}

	for (i = 0; i < n / 5; i++)
	{
		he = heap.heap[rand() % n];
		cout << "updating " << he->heapIdx << endl;
		he->cost = drand48();
		heap.Update(he);
	}
	
	for (i = 0; i < n / 5; i++)
	{
		he = heap.heap[rand() % heap.NumItems()];
		cout << "removing " << he->heapIdx << endl;
		heap.Delete(he);
	}
	
	for (i = 0; i < heap.NumItems(); i++)
		if (heap.heap[i]->heapIdx != i)
			cout << "bad heapIdx at " << i << " : " << heap.heap[i]->heapIdx << endl;

	cout << heap.NumItems() << " items in heap:" << endl;
	i = 0;
		
	while (he = heap.RemoveMax())
	{
		cout << i++ << " > " << he->cost << endl;
		delete he;
	}

}

Void TestObjArray()
{


}

#ifdef UNFINISHED
Void TestSplayTree()
{
    /* A sample use of these functions.  Start with the empty tree,         */
    /* insert some stuff into it, and then delete it                        */
    Tree * root;
    int i;
    root = NULL;               /* the empty tree */
    size = 0;
    for (i = 0; i < 1024; i++)
    {
        root = insert((541 * i) & (1023), root);
    }
    for (i = 0; i < 1024; i++)
    {
        root = delete((541 * i) & (1023), root);
    }
    printf("size = %d\n", size);
}
#endif

Int main(Int argc, Char *argv[])
{
	FileName test;
	
	TestEnviron();
	TestList();	
	TestString();
	TestInstArray();
	TestHash();
	TestFileName();
	TestHeap();
		
	return(0);
}
