/*
	File:			InstArray.cc

	Function:		Template definitions for InstArray.h

	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1996, Andrew Willmott

	Notes:			

*/

#include "cl/InstArray.h"

/*!
	/class InstArray

	This is a NArray instrumented so that
		- reads and writes are tracked
		- accesses are optionally written to a file. (can then look at 
		access patterns with gnuplot.)
		- accesses can optionally be accumulated into a counter array.
 */
 
InstArrayStats::InstArrayStats() :
		reads(0),
		writes(0),
		accesses(0),
		trace(0)
{

}

Void InstArrayStats::Reset()
{
	reads = 0;
	writes = 0;
}

Void InstArrayStats::StartTrace(const FileName &fname)
{
	trace = fopen(fname.GetPath(), "w");
}

Void InstArrayStats::StopTrace()
{
	fclose(trace);
	trace = 0;
}

Void InstArrayStats::Read(Int i)
{
	reads++;
	if (accesses)
		accesses[i]++;
	if (trace)
		fprintf(trace, "%d\n", i);
}

Void InstArrayStats::Write(Int i)
{
	writes++;
	if (accesses)
		accesses[i]++;
	if (trace)
		fprintf(trace, "%d\n", i);
}

Void InstArrayStats::Dump()
{
	cout << "reads             " << reads << endl;
	cout << "writes            " << writes << endl;
	cout << "total accesses    " << reads + writes << endl;
}
