/*
	File:			Timer.h

	Function:		Provides routines for timing under several different 
					architectures. You must define one of the following:
				
						UNIX_TIME		Use the time() and times() calls
						ANSI_TIME		Use the ansi clock() routine
						SGI_TIME		Use the SGI multimedia timing 
										routines (nano-second accuracy.)
						RUSAGE_TIME 	Use getrusage() system call 
		
	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1998, Andrew Willmott
 */

#ifndef __Timer__
#define __Timer__

#include "cl/Basics.h"

// Account for the morons who coded X11
#ifdef CurrentTime
#undef CurrentTime
#endif

class Timer
// all times in seconds. 
{
public:
	Timer() : startTime(0.0), stopTime(0.0), lapTime(0.0)
	{};

	Void	StartTimer();	// Starts timer
	Void	StopTimer();	// Stops timer
	Void	ContTimer();	// Continue timer
	Float	GetTimer();		// Returns time since timer was started

	Float	DeltaTime();	// returns time delta since DeltaTime
							// or StartTimer was last called

	virtual Float	CurrentTime() = 0;

protected:
	Float	startTime;
	Float	stopTime;
	Float	lapTime;
};

class ProgramTimer : public Timer
// measures elapsed program time.
{
public:
	ProgramTimer() : Timer(), addSystem(false)
	{};
	
	Float	CurrentTime();

	Bool	addSystem;
};

class WallClockTimer : public Timer
// measures wall clock time.
{
public:
	Float	CurrentTime();
};

#endif
