/*
	File:		Animator.cc

	Function:	Little command line interface to the Animation class.

	Author:		Andrew Willmott

	Notes:		
*/

#include "Animation.h"
#include "arg.h"

Int main(Int argc, Char *argv[])
{
	Animation 	anim;
	Double		fps = 24.0;
	Int			startFrame = -1, endFrame = -1, frameStep = 1;
	Int			frame;
	GCLReal		time;
	Char		*file1;
	
	// command-line options
	
	if (arg_parse(argc, argv, 
			"", 										"usage: anim [options]",
			
			"%S",			&file1,						"animation file",
			"-fps %F",		&fps,						"set frames per sec [default 24]",
			"-start %d", 	&startFrame,				"start frame [1]",
			"-end %d", 		&endFrame,					"end frame [24]",
			"-step %d", 	&frameStep,					"frame step [1]",
			"-d", 			ARG_FLAG(&anim.addDash),	"add dashes to variables",
			"-q", 			ARG_FLAG(&anim.quiet),		"quiet",
			"-o %S", 		&anim.tag,					"output tag_<frame>",

			0) < 0)
		exit(1);

	anim.ParseAnimFile(file1);
	if (startFrame < 0)
	{
		startFrame = 1;
		if (endFrame < 0)
			endFrame = Int(anim.length * fps);
	}
	else if (endFrame < 0)
		endFrame = startFrame;
				
	for (frame = startFrame; frame <= endFrame; frame += frameStep)
	{
		time = (frame - 1) / fps;
		anim.DumpAvars(time, frame); 
	}
}

