/*
	File:		GCLApp.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

// XXX
#include "gcl/GCLApp.h"
#include "gcl/Avars.h"
#include "gcl/Readers.h"
#include <stdlib.h>

#define DBG_COUT if (0) cerr


// --- GCLApp --------------------------------------------------------------


Char			**GCLApp::files;
Int				GCLApp::numFiles;
StrConstArray	GCLApp::avarNames;
ScalarList		GCLApp::avarVals;

GCLApp::GCLApp() : 
#ifndef GCL_NO_GL
    itsScenePane(0),
#endif
    itsCamera(0),
    frame(1),
    time(0.0),
    fps(24.0),
    anim(0),
    avarList(0)
{
}

Void GCLApp::ReadScene(FileName &sceneFile)
{
	scene = SceneReader::Load(sceneFile);
	if (!scene)
		return;

	if (zUp)
		scene->Apply(Rotation(vl_x, vl_halfPi));
	else if (zDown)
		scene->Apply(Rotation(vl_x, -vl_halfPi));
	
	if (!noScale)
		scene->Normalise();
}

Void GCLApp::GetFileArgs(Int argc, Char *argv[])
{
	// Save away a pointer to the list of file names
	// I've checked the source of libarg.c -- it
	// doesn't free anything, so this is safe.

	files = argv;
	numFiles = argc;
}

Void GCLApp::GetAvarArgs(Int argc, Char *argv[])
{
	Int		i;
	
	for (i = 0; i < argc; )
	{
		StrConst avarName = argv[i++];
		
		DBG_COUT << "looking at " << avarName << " suff = " << avarName.Suffix(2) << endl;

		if (avarName.Suffix(2) == "_c" || avarName.Suffix(2) == "_v")
		{
			String stub;

			stub = avarName.Prefix(-2);

			DBG_COUT << "got vector type " << stub << endl;

			// memory leaks here, but not worth bothering about.

			avarNames.Append(String().Printf("%s_x", stub.CString()).Detach());
			avarVals.Append(atof(argv[i++]));
			avarNames.Append(String().Printf("%s_y", stub.CString()).Detach());
			avarVals.Append(atof(argv[i++]));
			if (avarName.Suffix(1) == "v")
			{
				avarNames.Append(String().Printf("%s_z", stub.CString()).Detach());
				avarVals.Append(atof(argv[i++]));
			}
		}
		else
		{
			avarNames.Append((Char *) avarName.CString());
			avarVals.Append(atof(argv[i++]));
		}		
	}

	if (i != argc)
	{
		cerr << "Error: wrong number of avar arguments." << endl;
		exit(1);
	}
}

Void GCLApp::SetAvars(scScenePtr scene)
{
	Int				i;
	CreateAvarList	theAvarList;
	GCLReal			zoom = 1.0;	
	Coord			rot(vl_0);
	Vector			trans(vl_0);
	Bool			setCamParams = false;
	
	frame = -1;
	for (i = 0; i < avarNames.NumItems(); i++)
		cout << "avar " << avarNames[i] << " = " << avarVals[i] << endl;
		
	scene->ApplyAction(theAvarList);

#ifdef DEBUG
	cout << "scene avars:" << endl;
	for (i = 0; i < theAvarList.avarList->NumItems(); i++)
		cout << theAvarList.avarList->Item(i).name << " = " 
			<< theAvarList.avarList->Item(i).value << endl;
#endif

	scene->Set(theAvarList.avarList);
	
	for (i = 0; i < avarNames.NumItems(); i++)
	{
		StrConst avarName = avarNames[i];

		// XXX this is a hack for now: later camera params should be part of
		// actual avar mechanism
		if (avarName == "frame")
			frame = (Int) avarVals[i];
		else if (avarName == "time")
			time = avarVals[i];
		else if (avarName.Prefix(-1) == "cam_trans_")
		{
			trans[avarName.Suffix(1)[0] - 'x'] = avarVals[i];
			setCamParams = true;
		}
		else if (avarName.Prefix(-1) == "cam_rot_")
		{
			rot[avarName.Suffix(1)[0] - 'x'] = avarVals[i];
			setCamParams = true;
		}
		else if (avarName == "zoom")
		{
			zoom = avarVals[i];
			setCamParams = true;
		}
		else if (!theAvarList.avarList->SetAvar(avarNames[i], avarVals[i]))
			cerr << "No such avar in scene: " << avarNames[i] 
				 << " (ignoring) " << endl;
	}

	if (setCamParams && itsCamera)
	{
		itsCamera->SetSceneOrient(
			Rotation(vl_x, rot[0] * vl_pi)
			* Rotation(vl_y, rot[1] * vl_pi));

		itsCamera->SetSceneOffset(Shift(trans * 4.0));
		itsCamera->SetZoom(zoom);
		itsCamera->SetAspect(GCLReal(xsize) / GCLReal(ysize));
	}
#ifndef GCL_NO_GL
	if (setCamParams && itsScenePane)
	{
		itsScenePane->SetViewAngles(rot[0], rot[1]);
		itsScenePane->SetTranslate(trans[0], trans[1], trans[2]);
		itsScenePane->SetZoom(zoom);
	}
#endif
}

Void GCLApp::SetAvarsFromAnim(scScenePtr scene)
{
	if (!avarList)
	{
		avarList = (scAvarList*) scene->FindFirst(aAvarList);

		if (!avarList)
		{
			CreateAvarList	theAvarList;

			cout << "creating avar list" << endl;
			scene->ApplyAction(theAvarList);
			scene->Set(theAvarList.avarList);
			avarList = theAvarList.avarList;
		}
	}
	
	time = (frame - 1) / fps;
	if (anim)
		anim->SetAvarList(this, time);
}
