/*
	File:		GLXPane.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

#include "gcl/GCLConfig.h"

#ifndef GCL_NO_GL

#include "gcl/GLXPane.h"

// --- GLXPane class ----------------------------------------------------------

//	Attributes for a normal, 24-bit, z-buffered window
 
static Int attributeList[] = 
{ GLX_RGBA, GLX_DEPTH_SIZE, 12, GLX_RED_SIZE, 1, None };

static Int dblAttributeList[] = 
{ GLX_RGBA, GLX_DEPTH_SIZE, 12, GLX_RED_SIZE, 1, GLX_DOUBLEBUFFER, None };

XVisualInfo	*GLXPane::sGLXVisualInfo = 0;
XVisualInfo	*GLXPane::sGLXDblVisualInfo = 0;

GLXPane::GLXPane(Bool db) :
	GLRenderer(),
	XEventPane()
{
	doubleBuffered = db;
}

GLXPane::~GLXPane()
{
}

Void GLXPane::Init()
//	Attach GL context to a drawing environment
{
	XVisualInfo		*theVisualInfo;
	
	XPane::Init();

	theVisualInfo = GetVisualInfo();

	context = glXCreateContext(xgs->display, theVisualInfo, 0, GL_TRUE);
	glXMakeCurrent(xgs->display, paneXID, context);
	
	glWidth = width; 
	glHeight = height;

	GLRenderer::Init();
}

XVisualInfo *GLXPane::GetVisualInfo()
{
	if (!sGLXVisualInfo)
	{
		sGLXVisualInfo = 
			glXChooseVisual(xgs->display, DefaultScreen(xgs->display), attributeList);

		if (!sGLXVisualInfo)
			_Error("Can't obtain true-colour/z-buffered display for GL");
		
		sGLXDblVisualInfo = glXChooseVisual(xgs->display, DefaultScreen(xgs->display),
										dblAttributeList);	

		if (!sGLXDblVisualInfo)
		{
			_Warning("Can't get double-buffered display");
		
			// If we can't grab a double-buffered display, default to
			// single-buffered.
			sGLXDblVisualInfo = sGLXVisualInfo;	
		}
	}

	if (doubleBuffered)
		return(sGLXDblVisualInfo);
	else
		return(sGLXVisualInfo);
}

Void GLXPane::MakeCurrent()
{
	if (!xgs)
		_Error("Tried to manipulate a non-attached GLXPane");

	if (!glXMakeCurrent(xgs->display, paneXID, context))
		_Error("Couldn't swap graphics context");
}

Void GLXPane::Show()
{
	GLRenderer::Show();
	
	if (doubleBuffered)
		glXSwapBuffers(xgs->display, paneXID);
}


// --- GLXOffscreenPane class -------------------------------------------------

XVisualInfo	*GLXOffscreenPane::sGLXVisualInfo = 0;

GLXOffscreenPane::GLXOffscreenPane() : 
	XOffscreenPane(),
	GLRenderer()
{
}

Void GLXOffscreenPane::Init()
//	Attach GL context to a drawing environment
{
	XPane::Init();
	
	pixmap = glXCreateGLXPixmap(xgs->display, sGLXVisualInfo, paneXID);
	context = 
		glXCreateContext(xgs->display, sGLXVisualInfo, 0, GL_FALSE);

	glXMakeCurrent(xgs->display, pixmap, context);

	glWidth = width; 
	glHeight = height;
	
	GLRenderer::Init();
}

Void GLXOffscreenPane::MakeCurrent()
{
	if (!xgs)
		_Error("Tried to manipulate a non-attached GLXOffscreenPane");

	if (!glXMakeCurrent(xgs->display, pixmap, context))
		_Error("Couldn't swap graphics context");
}

XVisualInfo *GLXOffscreenPane::GetVisualInfo()
{
	if (!sGLXVisualInfo)
	{
		sGLXVisualInfo = 
			glXChooseVisual(xgs->display, DefaultScreen(xgs->display), attributeList);

		if (!sGLXVisualInfo)
			_Error("Can't obtain true-colour/z-buffered display for GL");
	}

	return(sGLXVisualInfo);
}

GLXOffscreenPane::~GLXOffscreenPane()
{
	if (xgs)
		glXDestroyGLXPixmap(xgs->display, pixmap);
}

#endif
