/*
	File:			SceneLang.h

	Function:		Allows run-time definition of scenes in the standard
					imperative manner.
					
	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1998, Andrew Willmott
 */

#ifndef __SceneLang__
#define __SceneLang__

#include "gcl/Geometry.h"
#include "gcl/Scene.h"
#include "gcl/SLLibrary.h"

Void		slInit();
			// Call before using any other sl<function> calls

scObject 	*slCurrent();		
			// Returns current object
scScenePtr  slCurrentGroup();	
			// Returns current group

scScenePtr	slBeginObject(const Char *name);
Void		slBeginObject(scScenePtr object);
Void 		slEndObject();

Void 		slPointList();
Int 		slPoint(const Point &p);

Void 		slColourList();
Int 		slColour(const Colour &c);

Void 		slNormalList();
Int 		slNormal(const Vector &n);

Void 		slCoordList();
Int 		slCoord(const Coord &c);
Void		slCoords(const CoordList &cl);

Void 		slPointIndexList();
Void 		slColourIndexList();
Void 		slCoordIndexList();
Void 		slNormalIndexList();

Void 		slPointIndex(Int i);
Void 		slColourIndex(Int i);
Void 		slNormalIndex(Int i);
Void 		slCoordIndex(Int i);

Void 		slPointIndexes(const IndexList &il);
Void 		slColourIndexes(const IndexList &il);
Void 		slNormalIndexes(const IndexList &il);
Void 		slCoordIndexes(const IndexList &il);

Void		slBeginFaces();
Void		slFace();
Void		slEndFaces();

Void 		slEmittance(const Colour &c);
Void		slTexture(StrConst filename);

Void		slPoly();
Void		slCamera();
Void 		slTransform(const Transform &t);

Void 		slApply(const Transform &t);

Void 		slObject(const Char *name);
Bool		slObjectExists(const Char *name);
Void 		slObject(scObject *object);
Void 		slObjectFile(StrConst filename);
Void 		slAttribute(scAttribute *attrPtr);

scAttribute slGetAttribute(Int attr);
scScenePtr 	slGetObject(Char* name);

Void		slAddToLibrary(scGroup *newGroup);
SLLibrary	*slSwapLibrary(SLLibrary *newLib);
Void		slSetLibrary(SLLibrary *library);
SLLibrary 	*slGetLibrary();

Void		slIndex(scAttributeID id, Int i);
Void		slIndexList(scAttributeID id);

/*
	Example:

	slPointList(); slPoint(a); slPoint(b); slPoint(c); slPoint(d);
	slBeginFaces();
	slIndexes(Indexes(0, 1, 2, IDX_END));
	slFace();
	slIndexes(Indexes(2, 3, 0, IDX_END));
	slFace();
	slEndFaces();
*/


#endif
