/*
	File:			ScenePane.h

	Function:		'ScenePane' provides a pane which you can attach a scene
					to. The pane then lets the user manipulate the view of the
					scene.

	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1996, Andrew Willmott
 */

#ifndef __ScenePane__
#define __ScenePane__

#include "gcl/GLXPane.h"
#include "gcl/SceneObjects.h"
#include "gcl/MRModel.h"

class scBaseGrid;

class ScenePane : public GLXPane
{
public:	
	ScenePane(Bool doubleBuf = true);
	
	virtual Void 	SetScene(scScenePtr scene);
	
	virtual Void 	Redraw();				
	// Force redraw of scene.
	Void 			ViewFrom(Point &position, Vector &normal);
	// Change viewing point
	Void			Restore();

	virtual Void	StartDrag(Coord start, Int modifiers);
	virtual Void	Drag(Coord current, Int modifiers);
	// Mouse is being dragged from start, currently at current.
	virtual Void	EndDrag(Coord end, Int modifiers);
	// Mouse was dragged from start to end.

	Void			ComplexityDrag();
	Void			SceneRotDrag();
	Void			SceneTransDrag();
	Void			SceneTransDrag2D();
			
	Void			HandleKey(Char c);
	Void			HandleExpose();
	Void			TrackMouse(Int x, Int y, Int modifiers);

	Void			SetGrid(Bool on);

	Camera*			ItsCamera() { return(itsCamera); };
	scScenePtr		ItsScene() { return(itsScene); };

	Void			SetCameraParams();
	Void			SetViewAngles(GCLReal xRot, GCLReal yRot)
					{ rotX = xRot; rotY = yRot; };
	Void			SetTranslate(GCLReal tx, GCLReal ty, GCLReal tz)
					{ transX = tx; transY = ty; transZ = tz; };
	Void			SetZoom(GCLReal z)
					{ zoom = z; };
	Void			PrintCameraParams(Bool animFormat = false);
	
	MRModelsFinder	models;

protected:
	Coord			dragStart;
	Int				modifiersStart;
	Coord			offset;
	GCLReal			baseErr;

	GCLReal			rotX, rotY;
	GCLReal			transX, transY, transZ;
	GCLReal 		zoom;
	
	GCLReal			tRotX, tRotY;
	GCLReal			tTransX, tTransY, tTransZ;
	GCLReal 		tZoom;
	
	Bool			wire;
	scScenePtr		itsScene;
	scBaseGrid		*grid;
	scCamera		*itsCamera;
	Camera			savedCam;
	Bool			isSavedCam;
	Bool            gridShown;
};

class ScenePane2D : public ScenePane
{
public:
	Void			Drag(Coord current, Int modifiers);
};

#endif
