/*
	File:		VertexJoin.h
	
	Function:	Join up disconnected mesh.
	
	Author:		Andrew Willmott	

	Usage:		Call Init() with the bounds of the scene, and current number
				of vertices. Then call AddVertex() to get new ids for each
				vertex in turn. When finished call GetFinalVertexList() to
				retrieve the new list of vertices.
*/

#ifndef __VertexJoin__
#define __VertexJoin__

#include "gcl/Geometry.h"

class VertexJoin
{
public:
	VertexJoin() : vertices(0), grid(0) {};
	
	Void		Init(const Point &min, const Point &max, Int numVertices);

	Int			AddVertex(const Point &where, Int id = 0);
				// add the vertex: returns the index this vertex will have in 
				// the final vertex list. If id is specified, vertices with
				// different ids will not be merged.
	
	PointList	*GetFinalVertexList();	

protected:
	Bool		MatchVertex(const Point &a, const Point &b);
				// are these the same point in space?

	Int			*grid;		// spatial grid storage
	Point		min, max;
	Int			cells[3];	// grid dimensions
	PointList	*vertices;	// list of new vertices
	IndexList	nextList;	// for list of points
	IndexList	idList;		// list of vertex ids
							// (vertices with different IDs won't be merged)
};

#endif
