/*
	File:			XGraphicsSystem.h

	Function:		Provides an XWindows-based implementation of the
					Renderer object. See Renderer.h
					
	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1996, Andrew Willmott
 */

#ifndef __XGraphicsSystem__
#define __XGraphicsSystem__

#include <iostream.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#undef Bool			
// The X cruft #defines Bool to be nothing(!), so we undef it.
// Oh for a world without X Windows.

#include "gcl/GCLConfig.h"
#include "gcl/Image.h"
#include "cl/PtrArray.h"

class XPane;
class XEventPane;
class XOffscreenPane;


// --- Graphics System class --------------------------------------------------

// List of all panes under our control.
typedef PtrArray<XEventPane> PaneList; 

class XGraphicsSystem
{
public:
					XGraphicsSystem();
	virtual			~XGraphicsSystem();
	 
	Void			CreateWindow(XEventPane *xpane, 
						StrConst title = "OpenGL window", Int width = 400,
						Int height = 400);
	Void			CreateSubwindow(XEventPane *xpane, 
									Window parent, Int x = 0,
						Int y = 0, Int width = 400, Int height = 400);
	Void			CreateOffscreen(XOffscreenPane *xpane,
						Int width = 400, Int height = 400);

	virtual Void 	Spin();
					// non blocking event handler: returns after a pass through
					// the event queue
	virtual Void 	Run();
					// blocking event handler
	virtual Bool 	RunFor(Float time);
					// blocking event handler: returns after 'time' seconds.
					// returns true if the handler is done.
	virtual Void 	SignalDone();
					// tell blocking event handler to quit.


	virtual Void	GetMouse(XEventPane *xpane, 
							 Int *x, Int *y, UInt *keyState);
	
	Void			DumpVisualInfo();


	PaneList		paneList;
	Display			*display;
	GC				theGC;
#ifdef GCL_XSHM
	Bool			sharedOn;
	Bool			remoteChecked;
#endif

protected:
	static Bool		finished;
	Long			itsEventsMask;
	Bool			hasGC;
};


// --- X window panes ---------------------------------------------------------

class XPane
{
public:
						XPane();
	virtual				~XPane();
					
	virtual Void 		Init();
	virtual XVisualInfo	*GetVisualInfo();

	Void				CopyFrom(XPane &from);
	Void				PutImage(Image &img, Int x = 0, Int y = 0);
	
	XID					paneXID;
	Int					width, height;
	XGraphicsSystem		*xgs;
	Bool				doubleBuffered;

protected:
	XImage 				*ImageToXImage(Image &img);

	static XVisualInfo	*sVisualInfo;
};

class XOffscreenPane : public XPane
{
public:
						XOffscreenPane();
						~XOffscreenPane();
};

enum WM_Modifier 
{
	wmShift		= 0x0001,
	wmControl	= 0x0002,
	wmAlt		= 0x0004,
	wmUndef1  	= 0x0008,
	wmButton1  	= 0x0010,
	wmButton2  	= 0x0020,
	wmButton3  	= 0x0040,
	wmButtons	= 0x0070,
	wmAll		= 0x00FF
};

class XEventPane : public XPane
//	can handle events...
{
public:

					XEventPane(Bool doubleBuffer = 0) : 
						XPane() {doubleBuffered = doubleBuffer;};

					~XEventPane();
	
	virtual Void	XHandleEvent(XEvent *event);
	Void			XGetMouse(Int *x, Int *y, UInt *keyState);

	// nice interface

	virtual Void	HandleKey(Char key);
	virtual Void	TrackMouse(Int x, Int y, Int modifiers);
	virtual Void	HandleExpose();
	Void			GetMouse(Int &x, Int &y, Int &modifiers);

	Void			Resize(Int x, Int y);
	Void			Hide();
	Void			Show();
	Void			SetTitle(StrConst title);
};

class XBackedPane : public XEventPane
// mirrors another pane, with screen update when expose events are received.
{
public:
					XBackedPane();

	Void			HandleExpose();

	Void			Attach(XPane *pane);
	
protected:
	XPane			*backPane;
};

#endif
