/*
	File:			SubVec.cc

	Function:		Implements SubVec.h

	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1996, Andrew Willmott

	Notes:			

*/

#include "vl/SubVec.h"
#include "vl/Vec.h"


// --- Vector Memory Management -----------------------------------------------


TSubVec::TSubVec(Int n, Int span, TVReal data[]) : elts(n), span(span),
	data(data)
{
}

TSubVec::TSubVec(const TSubVec &v) : elts(v.elts), span(v.span), data(v.data)
{
}

TSubVec::TSubVec(const TVec &v) : elts(v.Elts()), span(1), data(v.Ref())
{
}

TSubVec &TSubVec::operator = (const TSubVec &v)
{
	Assert(Elts() == v.Elts(), "(SubVec::=) Vector sizes don't match");
	Int i;
	
	for (i = 0; i < Elts(); i++)
		SELF[i] = v[i];
		
	return(SELF);
}

TSubVec &TSubVec::operator = (const TVec &v)
{
	Assert(Elts() == v.Elts(), "(SubVec::=) Vector sizes don't match");
	Int i;
	
	for (i = 0; i < Elts(); i++)
		SELF[i] = v[i];
		
	return(SELF);
}


// --- SubVec In-Place operators ----------------------------------------------


TSubVec &operator += (TSubVec &a, const TSubVec &b)
{
	Assert(a.Elts() == b.Elts(), "(Vec::+=) vector sizes don't match");	

	Int		i;
	
	for (i = 0; i < a.Elts(); i++) 
		a[i] += b[i];
	
	return(a);
}

TSubVec &operator -= (TSubVec &a, const TSubVec &b)
{
	Assert(a.Elts() == b.Elts(), "(Vec::-=) vector sizes don't match");	

	Int		i;
	
	for (i = 0; i < a.Elts(); i++) 
		a[i] -= b[i];
		
	return(a);
}

TSubVec &operator *= (TSubVec &a, const TSubVec &b)
{
	Assert(a.Elts() == b.Elts(), "(Vec::*=) Vec sizes don't match");

	Int		i;
	
	for (i = 0; i < a.Elts(); i++) 
		a[i] *= b[i];
	
	return(a);
}

TSubVec &operator *= (TSubVec &v, TVReal s)
{
	Int		i;
	
	for (i = 0; i < v.Elts(); i++) 
		v[i] *= s;
	
	return(v);
}

TSubVec &operator /= (TSubVec &a, const TSubVec &b)
{
	Assert(a.Elts() == b.Elts(), "(Vec::/=) Vec sizes don't match");

	Int		i;
	
	for (i = 0; i < a.Elts(); i++) 
		a[i] /= b[i];
	
	return(a);
}

TSubVec &operator /= (TSubVec &v, TVReal s)
{
	Int		i;
	
	for (i = 0; i < v.Elts(); i++) 
		v[i] /= s;
	
	return(v);
}
